@@ -1900,7 +1900,7 @@
                    :state (cond-> state column-validation-message (conj column-validation-message))
                    :cut true})
 
-                [["extract" column [subquery-name :guard (complement #{"not" "group_by"}) _]]]
+                [["extract" column [subquery-name :guard (complement #{"not" "group_by" "or" "and"}) _]]]
                 (let [underscored-subquery-name (utils/dashes->underscores subquery-name)
                       error (if (contains? (set (keys user-name->query-rec-name)) underscored-subquery-name)
                               (i18n/trs "Unsupported subquery `{0}` - did you mean `{1}`?" subquery-name underscored-subquery-name)
