@@ -4,6 +4,7 @@
    Functions that handle conversion of reports from wire format to
    internal PuppetDB format, including validation."
   (:require [schema.core :as s]
+            [clojure.set :as set]
             [puppetlabs.puppetdb.schema :as pls]
             [puppetlabs.puppetdb.utils :as utils]
             [com.rpl.specter :as sp]))
@@ -12,6 +13,7 @@
 
 (def event-wireformat-schema
   {:status s/Str
+   :corrective_change (s/maybe s/Bool)
    :timestamp pls/Timestamp
    :property (s/maybe s/Str)
    :new_value (s/maybe pls/JSONable)
@@ -26,6 +28,7 @@
    :file (s/maybe s/Str)
    :line (s/maybe s/Int)
    :containment_path [s/Str]
+   :corrective_change (s/maybe s/Bool)
    :events [event-wireformat-schema]})
 
 (def metric-wireformat-schema
@@ -50,21 +53,36 @@
    :start_time pls/Timestamp
    :end_time pls/Timestamp
    :producer_timestamp pls/Timestamp
+   :producer (s/maybe s/Str)
+   :corrective_change (s/maybe s/Bool)
    :resources [resource-wireformat-schema]
    :noop (s/maybe s/Bool)
+   :noop_pending (s/maybe s/Bool)
    :transaction_uuid (s/maybe s/Str)
+   :catalog_uuid (s/maybe s/Str)
+   :code_id (s/maybe s/Str)
+   :cached_catalog_status (s/maybe s/Str)
    :metrics [metric-wireformat-schema]
    :logs [log-wireformat-schema]
    :environment s/Str
    :status (s/maybe s/Str)})
 
+(def report-v7-wireformat-schema
+ (-> report-wireformat-schema
+     (dissoc :producer :noop_pending :corrective_change)))
+
+(def report-v6-wireformat-schema
+  (-> report-v7-wireformat-schema
+      (dissoc :catalog_uuid :cached_catalog_status :code_id)))
+
 (def resource-event-v5-wireformat-schema
   (-> resource-wireformat-schema
       (dissoc :skipped :events)
-      (merge event-wireformat-schema)))
+      (merge event-wireformat-schema)
+      (dissoc :corrective_change)))
 
 (def report-v5-wireformat-schema
-  (-> report-wireformat-schema
+  (-> report-v6-wireformat-schema
       (dissoc :resources)
       (assoc :resource_events [resource-event-v5-wireformat-schema])))
 
@@ -93,6 +111,7 @@
    :resource_title s/Str
    :resource_type s/Str
    :timestamp pls/Timestamp
+   :corrective_change (s/maybe s/Bool)
    :containing_class (s/maybe s/Str)
    :containment_path (s/maybe [s/Str])
    :property (s/maybe s/Str)
@@ -126,20 +145,35 @@
    (s/optional-key :start_time) pls/Timestamp
    (s/optional-key :end_time) pls/Timestamp
    (s/optional-key :producer_timestamp) pls/Timestamp
+   (s/optional-key :producer) (s/maybe s/Str)
+   (s/optional-key :corrective_change) (s/maybe s/Bool)
    (s/optional-key :noop) (s/maybe s/Bool)
+   (s/optional-key :noop_pending) (s/maybe s/Bool)
    (s/optional-key :report_format) s/Int
    (s/optional-key :configuration_version) s/Str
-   (s/optional-key :resources) resources-expanded-query-schema
+   (s/optional-key :resources) (s/maybe resources-expanded-query-schema)
    (s/optional-key :metrics) metrics-expanded-query-schema
    (s/optional-key :logs) logs-expanded-query-schema
    (s/optional-key :resource_events) resource-events-expanded-query-schema
-   (s/optional-key :transaction_uuid) s/Str
+   (s/optional-key :transaction_uuid) (s/maybe s/Str)
+   (s/optional-key :catalog_uuid) (s/maybe s/Str)
+   (s/optional-key :code_id) (s/maybe s/Str)
+   (s/optional-key :cached_catalog_status) (s/maybe s/Str)
    (s/optional-key :status) (s/maybe s/Str)})
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Reports Query -> Wire format conversions
 
 (pls/defn-validated resource-events-query->wire-v5 :- [resource-event-v5-wireformat-schema]
+  [resource-events :- resource-events-expanded-query-schema]
+  (->> resource-events
+       :data
+       (map #(dissoc %
+                     :report :certname :containing_class :configuration_version
+                     :run_start_time :run_end_time :report_receive_time :environment
+                     :corrective_change))))
+
+(pls/defn-validated munge-resource-events-for-v8
   [resource-events :- resource-events-expanded-query-schema]
   (->> resource-events
        :data
@@ -147,15 +181,24 @@
                      :report :certname :containing_class :configuration_version
                      :run_start_time :run_end_time :report_receive_time :environment))))
 
-(pls/defn-validated report-query->wire-v5 :- report-v5-wireformat-schema
-  [report :- report-query-schema]
+(defn generic-query->wire-transform
+  "Dissociate query-only and pe-only fields and replace href fields with the
+   corresponding data key."
+  [report]
   (-> report
       (dissoc :hash :receive_time :resources)
-      (update :resource_events resource-events-query->wire-v5)
       (update :metrics :data)
       (update :logs :data)))
 
-(defn reports-query->wire-v5 [reports]
+(defn report-query->wire-v5
+  [report]
+  (-> report
+      generic-query->wire-transform
+      (dissoc :noop_pending :corrective_change)
+      (update :resource_events resource-events-query->wire-v5)))
+
+(pls/defn-validated reports-query->wire-v5 :- [report-v5-wireformat-schema]
+  [reports :- [report-query-schema]]
   (map report-query->wire-v5 reports))
 
 (defn dash->underscore-report-keys [v5-report-or-older]
@@ -164,26 +207,48 @@
        (sp/transform [:resource_events sp/ALL sp/ALL]
                      #(update % 0 utils/dashes->underscores))))
 
-(defn- resource-event-v5->resource
+(defn- resource-event->resource
   [resource-event]
   (-> resource-event
       (select-keys [:file :line :timestamp :resource_type :resource_title :containment_path])
       (assoc :skipped (= "skipped" (:status resource-event)))))
 
-(defn resource-events-v5->resources
+(defn resource-events-wire->resources
   [resource-events]
   (vec
-   (for [[resource resource-events] (group-by resource-event-v5->resource resource-events)
-         :let [events (mapv #(dissoc % :file :line :resource_type :resource_title :containment_path) resource-events)]]
-     (assoc resource :events events))))
+    (for [[resource resource-events] (group-by resource-event->resource resource-events)
+          :let [events (mapv #(-> %
+                                  (dissoc :file :line :resource_type
+                                          :resource_title :containment_path)
+                                  (utils/assoc-when :corrective_change nil))
+                             resource-events)]]
+      (-> resource
+          (assoc :events events)
+          (utils/assoc-when :corrective_change nil)))))
 
-(defn wire-v5->wire-v6
+(defn wire-v7->wire-v8
+  [report]
+  (utils/assoc-when report
+                    :noop_pending nil
+                    :corrective_change nil
+                    :producer nil))
+
+(defn wire-v6->wire-v8
+  [{:keys [transaction_uuid] :as report}]
+  (-> report
+    (utils/assoc-when :catalog_uuid transaction_uuid
+                      :cached_catalog_status nil
+                      :code_id nil)
+    wire-v7->wire-v8))
+
+(defn wire-v5->wire-v8
   [report]
   (-> report
-      (update :resource_events resource-events-v5->resources)
-      (clojure.set/rename-keys {:resource_events :resources})))
+      (update :resource_events resource-events-wire->resources)
+      (set/rename-keys {:resource_events :resources})
+      wire-v6->wire-v8))
 
-(defn wire-v4->wire-v6
+(defn wire-v4->wire-v8
   [report received-time]
   (-> report
       dash->underscore-report-keys
@@ -191,23 +256,24 @@
              :logs nil
              :noop nil
              :producer_timestamp received-time)
-      wire-v5->wire-v6))
+      wire-v5->wire-v8))
 
-
-(defn wire-v3->wire-v6
+(defn wire-v3->wire-v8
   [report received-time]
   (-> report
       (assoc :status nil)
-      (wire-v4->wire-v6 received-time)))
+      (wire-v4->wire-v8 received-time)))
 
-(pls/defn-validated report-query->wire-v6 :- report-wireformat-schema
+(pls/defn-validated report-query->wire-v8 :- report-wireformat-schema
   [report :- report-query-schema]
   (-> report
-      report-query->wire-v5
-      wire-v5->wire-v6))
+      generic-query->wire-transform
+      (update :resource_events (comp resource-events-wire->resources
+                                     munge-resource-events-for-v8))
+      (set/rename-keys {:resource_events :resources})))
 
-(defn reports-query->wire-v6 [reports]
-  (map report-query->wire-v6 reports))
+(defn reports-query->wire-v8 [reports]
+  (map report-query->wire-v8 reports))
 
 (defn- resource->skipped-resource-events
   "Fabricate a skipped resource-event"
@@ -215,7 +281,7 @@
   (-> resource
       ;; We also need to grab the timestamp when the resource is `skipped'
       (select-keys [:resource_type :resource_title :file :line :containment_path :timestamp])
-      (merge {:status "skipped" :property nil :old_value nil :new_value nil :message nil})
+      (merge {:status "skipped" :property nil :old_value nil :new_value nil :message nil :corrective_change false})
       vector))
 
 (defn- resource->resource-events
