@@ -1,5 +1,6 @@
 (ns puppetlabs.puppetdb.utils
   (:require [puppetlabs.kitchensink.core :as kitchensink]
+            [puppetlabs.i18n.core :as i18n]
             [clojure.tools.logging :as log]
             [clojure.string :as string]
             [schema.core :as s]
@@ -33,7 +34,7 @@
   "Returns error message instructing the user to switch to JDK 1.7"
   []
   (attention-msg
-   (format "JDK 1.6 is no longer supported. PuppetDB requires JDK 1.7+, currently running: %s" kitchensink/java-version)))
+   (i18n/trs "JDK 1.6 is no longer supported. PuppetDB requires JDK 1.7+, currently running: {0}" kitchensink/java-version)))
 
 (defn println-err
   "Redirects output to standard error before invoking println"
@@ -263,6 +264,32 @@
       (apply assoc m new-kvs)
       m)))
 
+(defn maybe-strip-escaped-quotes
+  [s]
+  (if (and (> (count s) 1)
+           (string/starts-with? s "\"")
+           (string/ends-with? s "\""))
+    (subs s 1 (dec (count s)))
+    s))
+
+(defn parse-matchfields
+  [s]
+  (string/replace s #"match\((\".*\")\)" "$1"))
+
+(defn parse-indexing
+  [s]
+  (string/replace s #"\[(\d+)\]" ".$1"))
+
+(defn split-indexing
+  [path]
+  (flatten
+    (for [s path]
+      (if (re-find #"\[\d+\]$" s)
+        (-> s
+            (string/split #"(?=\[\d+\]$)")
+            (update 1 #(Integer/parseInt (subs % 1 (dec (count %))))))
+        s))))
+
 (defn str-schema
   "Function for converting a schema with keyword keys to
    to one with string keys. Doens't walk the map so nested
@@ -322,3 +349,20 @@
      result#))
 
 (def byte-array-class (Class/forName "[B"))
+
+(defmacro nil-on-failure
+  "Executes `body` and if an exception is thrown, returns nil"
+  [& body]
+  `(try
+     ~@body
+     (catch Exception _#
+       nil)))
+
+(defmacro noisy-future [& body]
+  `(future
+     (try
+       ~@body
+       (catch Exception ex#
+         (binding [*out* *err*]
+           (println ex#))
+         (throw ex#)))))
