@@ -1,38 +1,66 @@
 (ns puppetlabs.trapperkeeper.internal
-  (:import (clojure.lang Atom ExceptionInfo))
+  (:import (clojure.lang ExceptionInfo IFn IDeref))
   (:require [clojure.tools.logging :as log]
+            [beckon]
             [plumbing.graph :as graph]
+            [slingshot.slingshot :refer [throw+]]
             [puppetlabs.kitchensink.core :refer [add-shutdown-hook! boolean? cli!]]
             [puppetlabs.trapperkeeper.config :refer [config-service]]
             [puppetlabs.trapperkeeper.app :as a]
+            [puppetlabs.trapperkeeper.common :as common]
             [puppetlabs.trapperkeeper.services :as s]
-            [puppetlabs.kitchensink.core :as ks]))
+            [puppetlabs.kitchensink.core :as ks]
+            [schema.core :as schema]
+            [clojure.core.async :as async]
+            [clojure.core.async.impl.protocols :as async-prot]
+            [clojure.string :as string]))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Schemas
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(def LifeCycleTask
+  {:type (schema/enum :restart :shutdown :boot)
+   :task-function IFn})
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Private
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+;; This is (eww) a global variable that holds a reference to all of the running
+;; Trapperkeeper apps in the process. It can be used when connecting via nrepl
+;; to allow you to do useful things, and also may be used for other things
+;; (such as signal handling).
+(def tk-apps (atom []))
+
+(def max-pending-lifecycle-events 5)
 
 (defn service-graph?
   "Predicate that tests whether or not the argument is a valid trapperkeeper
   service graph."
   [service-graph]
   (and
-    (map? service-graph)
-    (every? keyword? (keys service-graph))
-    (every? (some-fn ifn? service-graph?) (vals service-graph))))
+   (map? service-graph)
+   (every? keyword? (keys service-graph))
+   (every? (some-fn ifn? service-graph?) (vals service-graph))))
 
 (defn validate-service-graph!
   "Validates that a ServiceDefinition contains a valid trapperkeeper service graph.
-  Returns the service definition on success; throws an IllegalArgumentException
+  Returns the service definition on success; throws an ::invalid-service-graph
   if the graph is invalid."
   [service-def]
   {:post [(satisfies? s/ServiceDefinition %)]}
   (if-not (satisfies? s/ServiceDefinition service-def)
-    (throw (IllegalArgumentException.
-             (str "Invalid service definition; expected a service "
-                  "definition (created via `service` or `defservice`); "
-                  "found: " (pr-str service-def)))))
+    (throw+ {:type ::invalid-service-graph
+             :message (str "Invalid service definition; expected a service "
+                           "definition (created via `service` or `defservice`); "
+                           "found: " (pr-str service-def))}))
   (if (service-graph? (s/service-map service-def))
     service-def
-    (throw (IllegalArgumentException. (str "Invalid service graph; service graphs must "
-                                           "be nested maps of keywords to functions.  Found: "
-                                           (s/service-map service-def))))))
+    (throw+ {:type ::invalid-service-graph
+             :message (str "Invalid service graph; service graphs must "
+                           "be nested maps of keywords to functions.  Found: "
+                           (s/service-map service-def))})))
 
 (defn parse-missing-required-key
   "Prismatic's graph compilation code throws `ExceptionInfo` objects if required
@@ -78,13 +106,13 @@
 
       (if (sequential? (:error data))
         (let [missing-services (keys (ks/filter-map
-                                       (fn [_ v] (= v 'missing-required-key))
-                                       (.error (first (:error data)))))]
+                                      (fn [_ v] (= v 'missing-required-key))
+                                      (.error (first (:error data)))))]
           (if (= 1 (count missing-services))
             (throw (RuntimeException.
-                     (format "Service '%s' not found" (first missing-services))))
+                    (format "Service '%s' not found" (first missing-services))))
             (throw (RuntimeException.
-                     (format "Services '%s' not found" missing-services)))))
+                    (format "Services '%s' not found" missing-services)))))
         (throw e)))))
 
 (defn compile-graph
@@ -111,16 +139,14 @@
       (handle-prismatic-exception! e))))
 
 
-(defn parse-cli-args!
+(schema/defn parse-cli-args! :- common/CLIData
   "Parses the command-line arguments using `puppetlabs.kitchensink.core/cli!`.
   Hard-codes the command-line arguments expected by trapperkeeper to be:
       --debug
       --bootstrap-config <bootstrap file>
       --config <.ini file or directory>
       --plugins <plugins directory>"
-  [cli-args]
-  {:pre  [(sequential? cli-args)]
-   :post [(map? %)]}
+  [cli-args :- [(schema/maybe schema/Str)]]
   (let [specs       [["-d" "--debug" "Turns on debug mode"]
                      ["-b" "--bootstrap-config BOOTSTRAP-CONFIG-FILE" "Path to bootstrap config file"]
                      ["-c" "--config CONFIG-PATH"
@@ -131,7 +157,7 @@
         required    []]
     (first (cli! cli-args specs required))))
 
-(defn run-lifecycle-fn!
+(schema/defn ^:always-validate run-lifecycle-fn!
   "Run a lifecycle function for a service.  Required arguments:
 
   * app-context: the app context atom; can be updated by the lifecycle fn
@@ -141,26 +167,25 @@
                        error message if an error occurs.
   * service-id: the id of the service that the lifecycle fn is being run on
   * s: the service that the lifecycle fn is being run on"
-  [app-context lifecycle-fn lifecycle-fn-name service-id s]
-  {:pre [(instance? Atom app-context)
-         (ifn? lifecycle-fn)
-         (string? lifecycle-fn-name)
-         (keyword? service-id)
-         (satisfies? s/Lifecycle s)]}
+  [app-context :- (schema/atom a/TrapperkeeperAppContext)
+   lifecycle-fn :- IFn
+   lifecycle-fn-name :- schema/Str
+   service-id :- schema/Keyword
+   s :- (schema/protocol s/Service)]
   (let [;; call the lifecycle function on the service, and keep a reference
         ;; to the updated context map that it returns
-        updated-ctxt  (lifecycle-fn s (get @app-context service-id {}))]
+        updated-ctxt  (lifecycle-fn s (get-in @app-context [:service-contexts service-id] {}))]
     (if-not (map? updated-ctxt)
       (throw (IllegalStateException.
-               (format
-                 "Lifecycle function '%s' for service '%s' must return a context map (got: %s)"
-                 lifecycle-fn-name
-                 (or (s/service-symbol s) service-id)
-                 (pr-str updated-ctxt)))))
+              (format
+               "Lifecycle function '%s' for service '%s' must return a context map (got: %s)"
+               lifecycle-fn-name
+               (or (s/service-symbol s) service-id)
+               (pr-str updated-ctxt)))))
     ;; store the updated service context map in the application context atom
-    (swap! app-context assoc service-id updated-ctxt)))
+    (swap! app-context assoc-in [:service-contexts service-id] updated-ctxt)))
 
-(defn run-lifecycle-fns
+(schema/defn run-lifecycle-fns
   "Run a lifecycle function for all services.  Required arguments:
 
   * app-context: the app context atom; can be updated by the lifecycle fn
@@ -174,9 +199,15 @@
                       (i.e. a service can be assured that any services it
                       depends on will have their corresponding lifecycle fn
                       called first.)"
-  [app-context lifecycle-fn lifecycle-fn-name ordered-services]
+  [app-context :- (schema/atom a/TrapperkeeperAppContext)
+   lifecycle-fn :- IFn
+   lifecycle-fn-name :- schema/Str
+   ordered-services :- a/TrapperkeeperAppOrderedServices]
   (try
     (doseq [[service-id s] ordered-services]
+      (log/debugf "Running lifecycle function '%s' for service '%s'"
+                  lifecycle-fn-name
+                  service-id)
       (run-lifecycle-fn! app-context
                          lifecycle-fn
                          lifecycle-fn-name
@@ -186,6 +217,88 @@
       (log/errorf t "Error during service %s!!!" lifecycle-fn-name)
       (throw t))))
 
+(schema/defn ^:always-validate initialize-lifecycle-worker :- (schema/protocol async-prot/Channel)
+  "Initializes a 'worker' which will listen for lifecycle-related tasks and perform
+  them on a background thread, to ensure that we aren't executing multiple lifecycle
+  tasks simultaneously."
+  [lifecycle-channel :- (schema/protocol async-prot/Channel)
+   shutdown-channel :- (schema/protocol async-prot/Channel)
+   shutdown-reason-promise :- IDeref]
+  (log/debug "Initializing lifecycle worker loop.")
+  (async/go-loop []
+    (let [[task chan] (async/alts!
+                       [shutdown-channel lifecycle-channel]
+                       :priority true)]
+      (schema/validate LifeCycleTask task)
+      (let [{:keys [type task-function]} task]
+        (condp #(contains? %1 %2) type
+          #{:shutdown}
+          (do
+            (try
+              (log/debug "Received shutdown command, shutting down services")
+              (async/close! shutdown-channel)
+              (async/close! lifecycle-channel)
+
+              (log/debug "Clearing lifecycle worker channels for shutdown.")
+              ;; drain the channels to ensure that there are
+              ;; no blocking puts, e.g. if a second shutdown request
+              ;; was queued simultaneously
+              (ks/while-let [msg (async/poll! shutdown-channel)]
+                            (log/debug "Shutdown in progress, ignoring message on shutdown channel:"
+                                       (:type msg)))
+              (ks/while-let [msg (async/poll! lifecycle-channel)]
+                            (log/debug "Shutdown in progress, ignoring message on main lifecycle channel:"
+                                       (:type msg)))
+
+              (task-function)
+
+              (log/debug "Service shutdown complete, exiting lifecycle worker loop")
+              (catch Exception e
+                (log/debug e "Exception caught during shutdown!")))
+            :done)
+
+          #{:boot :restart}
+          (do
+            (try
+              (log/debugf "Lifecycle worker executing %s lifecycle task." type)
+              (task-function)
+              (log/debugf "Lifecycle worker completed %s lifecycle task; awaiting next task." type)
+              (catch Exception e
+                (log/debug e "Exception caught in lifecycle worker loop")
+                (deliver shutdown-reason-promise
+                         {:cause :service-error
+                          :error e})))
+            (recur))
+
+          (do
+            (deliver shutdown-reason-promise
+                     {:cause :service-error
+                      :error (IllegalStateException. "Unrecognized lifecycle task:" task)})
+            (recur)))))))
+
+(defn restart-tk-apps
+  "Call restart on all tk apps."
+  [apps]
+  (log/debug "SIGHUP handler restarting TK apps.")
+  (doseq [app apps]
+    (let [{:keys [lifecycle-channel]} @(a/app-context app)
+          restart-fn #(a/restart app)]
+      (when-not (async/offer! lifecycle-channel
+                              {:type :restart
+                               :task-function restart-fn})
+        (log/warnf "Ignoring new SIGHUP restart requests; too many requests queued (%s)"
+                   max-pending-lifecycle-events)))))
+
+(defn register-sighup-handler
+  "Register a handler for SIGHUP that restarts all trapperkeeper apps. The
+  default handler terminates the process, so we always overwrite that. This
+  function is idempotent."
+  ([]
+   (register-sighup-handler @tk-apps))
+  ([apps]
+   (log/debug "Registering SIGHUP handler for restarting TK apps")
+   (reset! (beckon/signal-atom "HUP") #{(partial restart-tk-apps apps)})))
+
 ;;;; Application Shutdown Support
 ;;;;
 ;;;; The next section of this namespace
@@ -238,18 +351,17 @@
    (shutdown-on-error* shutdown-reason-promise app-context svc-id f nil))
   ([shutdown-reason-promise app-context svc-id f on-error-fn]
    (try
-     ; These would normally be pre-conditions; however, this function needs
-     ; to never throw an exception - since it is often called as a wrapper
-     ; around everything inside a `future`, it is important that this function
-     ; never throw anything (like an AssertionError from a pre-condition),
+     ; This schema check would normally be handled via schematizing the fn itself;
+     ; however, this function needs to never throw an exception - since it is often
+     ; called as a wrapper around everything inside a `future`, it is important
+     ; that this function ; never throw anything (like a schema validation error),
      ; since it is likely to just get lost in a `future`.  Instead,
      ; invalid arguments will simply cause the shutdown promise to be delivered.
-     (assert (instance? Atom app-context))
-     (assert (map? @app-context))
-     (assert (keyword? svc-id))
-     (assert (contains? @app-context svc-id))
-     (assert (ifn? f))
-     (assert ((some-fn nil? ifn?) on-error-fn))
+     (schema/validate a/TrapperkeeperAppContext @app-context)
+     (schema/validate schema/Keyword svc-id)
+     (assert (contains? (:service-contexts @app-context) svc-id))
+     (schema/validate IFn f)
+     (schema/validate (schema/maybe IFn) on-error-fn)
 
      (f)
      (catch Throwable t
@@ -267,7 +379,7 @@
     "Higher-order function to execute application logic and trigger shutdown in
     the event of an exception"))
 
-(defn- shutdown-service
+(schema/defn shutdown-service
   "Provides various functions for triggering application shutdown programatically.
   Primarily intended to serve application services, though TrapperKeeper also uses
   this service internally and therefore is always available in the application graph.
@@ -283,50 +395,45 @@
                            and trigger shutdown in the event of an exception
 
   For more information, see `request-shutdown` and `shutdown-on-error`."
-  [shutdown-reason-promise app-context]
+  [shutdown-reason-promise :- IDeref
+   app-context :- (schema/atom a/TrapperkeeperAppContext)]
   (s/service ShutdownService
     []
     (get-shutdown-reason [this] (when (realized? shutdown-reason-promise)
-                                      @shutdown-reason-promise))
+                                  @shutdown-reason-promise))
     (wait-for-shutdown [this] (deref shutdown-reason-promise))
     (request-shutdown [this]  (request-shutdown* shutdown-reason-promise))
     (shutdown-on-error [this svc-id f] (shutdown-on-error* shutdown-reason-promise app-context svc-id f))
     (shutdown-on-error [this svc-id f on-error] (shutdown-on-error* shutdown-reason-promise app-context svc-id f on-error))))
 
-(defn ordered-services?
-  "Predicate that validates that an object is an ordered list of services as required
-  for the app context map.  Each item in the list must be a tuple whose first element
-  is a service id (keyword) and whose second element is the service instance."
-  [os]
-  (or (nil? os)
-      (and
-        (sequential? os)
-        (every? vector? os)
-        (every? #(= (count %) 2) os)
-        (every? #(keyword? (first %)) os)
-        (every? #(satisfies? s/Lifecycle (second %)) os))))
-
-(defn shutdown!
+(schema/defn ^:always-validate shutdown!
   "Perform shutdown calling the `stop` lifecycle function on each service,
    in reverse order (to account for dependency relationships)."
-  [app-context]
-  {:pre [(instance? Atom app-context)
-         (let [ctxt @app-context]
-           (and (map? ctxt)
-                (ordered-services? (ctxt :ordered-services))))]}
+  [app-context :- (schema/atom a/TrapperkeeperAppContext)]
   (log/info "Beginning shutdown sequence")
-  (doseq [[service-id s] (reverse (@app-context :ordered-services))]
-    (try
-      (run-lifecycle-fn! app-context s/stop "stop" service-id s)
-      (catch Exception e
-        (log/error e "Encountered error during shutdown sequence"))))
-  (log/info "Finished shutdown sequence"))
+  (let [{:keys [ordered-services shutdown-channel lifecycle-worker]} @app-context
+        shutdown-fn (fn [] (doseq [[service-id s] (reverse ordered-services)]
+                             (try
+                               (run-lifecycle-fn! app-context s/stop "stop" service-id s)
+                               (catch Exception e
+                                 (log/error e "Encountered error during shutdown sequence")))))]
+    (log/trace "Putting shutdown message on shutdown channel.")
+    (async/>!! shutdown-channel {:type :shutdown
+                                 :task-function shutdown-fn})
+    ;; wait for the channel to send us the return value so we know it's done
+    (log/trace "Waiting for response to shutdown message from lifecycle worker.")
+    (if (not (nil? (async/<!! lifecycle-worker)))
+      (log/info "Finished shutdown sequence")
+      ;; else, the read from the channel returned a nil because it was closed,
+      ;; indicating that there was already a shutdown in progress, and thus the
+      ;; redundant shutdown request was ignored
+      (log/trace (str "Response from lifecycle worker indicates shutdown already in progress,"
+                      "ignoring additional shutdown attempt.")))))
 
-(defn initialize-shutdown-service!
+(schema/defn ^:always-validate initialize-shutdown-service! :- (schema/protocol s/ServiceDefinition)
   "Initialize the shutdown service and add a shutdown hook to the JVM."
-  [app-context shutdown-reason-promise]
-  {:pre [(instance? Atom app-context)]
-   :post [(satisfies? s/ServiceDefinition %)]}
+  [app-context :- (schema/atom a/TrapperkeeperAppContext)
+   shutdown-reason-promise :- IDeref]
   (let [shutdown-service        (shutdown-service shutdown-reason-promise
                                                   app-context)]
     (add-shutdown-hook! (fn []
@@ -400,21 +507,30 @@
 
 ;;;; end of shutdown-related functions
 
-(defn build-app*
+(schema/defn ^:always-validate build-app* :- (schema/protocol a/TrapperkeeperApp)
   "Given a list of services and a map of configuration data, build an instance
   of a TrapperkeeperApp.  Services are not yet initialized or started."
-  [services config-data shutdown-reason-promise]
-  {:pre  [(sequential? services)
-          (every? #(satisfies? s/ServiceDefinition %) services)
-          (map? config-data)]
-   :post [(satisfies? a/TrapperkeeperApp %)]}
-  (let [;; this is the application context for this app instance.  its keys
+  [services :- [(schema/protocol s/ServiceDefinition)]
+   config-data-fn :- IFn]
+  (let [shutdown-reason-promise (promise)
+        lifecycle-channel (async/chan max-pending-lifecycle-events)
+        shutdown-channel (async/chan)
+        ;; this is the application context for this app instance.  its keys
         ;; will be the service ids, and values will be maps that represent the
         ;; context for each individual service
-        app-context (atom {})
+        app-context (atom {:service-contexts {}
+                           :ordered-services []
+                           :services-by-id {}
+                           :lifecycle-channel lifecycle-channel
+                           :shutdown-channel shutdown-channel
+                           :lifecycle-worker (initialize-lifecycle-worker
+                                              lifecycle-channel
+                                              shutdown-channel
+                                              shutdown-reason-promise)
+                           :shutdown-reason-promise shutdown-reason-promise})
         service-refs (atom {})
         services (conj services
-                       (config-service config-data)
+                       (config-service config-data-fn)
                        (initialize-shutdown-service! app-context
                                                      shutdown-reason-promise))
         service-map (apply merge (map s/service-map services))
@@ -429,9 +545,10 @@
         ;; any further
         services-by-id @service-refs
         ordered-services (map (fn [[service-id _]] [service-id (services-by-id service-id)]) graph)]
-    (swap! app-context assoc :services-by-id services-by-id)
-    (swap! app-context assoc :ordered-services ordered-services)
-    (doseq [svc-id (keys services-by-id)] (swap! app-context assoc svc-id {}))
+    (swap! app-context assoc
+           :services-by-id services-by-id
+           :ordered-services ordered-services)
+    (doseq [svc-id (keys services-by-id)] (swap! app-context assoc-in [:service-contexts svc-id] {}))
     ;; finally, create the app instance
     (reify
       a/TrapperkeeperApp
@@ -439,7 +556,7 @@
       (a/service-graph [this] graph-instance)
       (a/app-context [this] app-context)
       (a/check-for-errors! [this] (throw-app-error-if-exists!
-                                    this))
+                                   this))
       (a/init [this]
         (run-lifecycle-fns app-context s/init "init" ordered-services)
         this)
@@ -448,31 +565,54 @@
         this)
       (a/stop [this]
         (shutdown! app-context)
-        this))))
+        this)
+      (a/restart [this]
+        (try
+          (run-lifecycle-fns app-context s/stop "stop" (reverse ordered-services))
+          (doseq [svc-id (keys services-by-id)] (swap! app-context assoc-in [:service-contexts svc-id] {}))
+          (run-lifecycle-fns app-context s/init "init" ordered-services)
+          (run-lifecycle-fns app-context s/start "start" ordered-services)
+          this
+          (catch Throwable t
+            (deliver shutdown-reason-promise {:cause :service-error
+                                              :error t})))))))
 
-(defn boot-services*
+(schema/defn ^:always-validate boot-services-for-app**
+  "Boots services for a TK app.  WARNING:  This should only ever be called
+  on the lifecycle-worker, presumably via `boot-services-for-app*`"
+  [result-promise :- IDeref
+   app :- (schema/protocol a/TrapperkeeperApp)]
+  (let [{:keys [shutdown-reason-promise]} @(a/app-context app)]
+    (try
+      (a/init app)
+      (a/start app)
+      (catch Throwable t
+        (deliver shutdown-reason-promise {:cause :service-error
+                                          :error t})))
+    (deliver result-promise app)))
+
+(schema/defn ^:always-validate boot-services-for-app* :- (schema/protocol a/TrapperkeeperApp)
+  [app :- (schema/protocol a/TrapperkeeperApp)]
+  (let [lifecycle-channel (:lifecycle-channel @(a/app-context app))
+        lifecycle-promise (promise)
+        boot-fn (partial boot-services-for-app** lifecycle-promise app)]
+    (async/>!! lifecycle-channel {:type :boot
+                                  :task-function boot-fn})
+    @lifecycle-promise
+    app))
+
+(schema/defn ^:always-validate boot-services* :- (schema/protocol a/TrapperkeeperApp)
   "Given the services to run and the map of configuration data, create the
   TrapperkeeperApp and boot the services.  Returns the TrapperkeeperApp."
-  [services config-data]
-  {:pre  [(sequential? services)
-          (every? #(satisfies? s/ServiceDefinition %) services)
-          (map? config-data)]
-   :post [(satisfies? a/TrapperkeeperApp %)]}
-  (let [shutdown-reason-promise (promise)
-        app                     (try
+  [services :- [(schema/protocol s/ServiceDefinition)]
+   config-data-fn :- IFn]
+  (let [app                     (try
                                   (build-app* services
-                                              config-data
-                                              shutdown-reason-promise)
+                                              config-data-fn)
                                   (catch Throwable t
                                     (log/error t "Error during app buildup!")
                                     (throw t)))]
-      (try
-        (a/init app)
-        (a/start app)
-        (catch Throwable t
-          (deliver shutdown-reason-promise {:cause :service-error
-                                            :error t})))
-      app)
-    )
+    (boot-services-for-app* app)
+    app))
 
 
