@@ -6,8 +6,8 @@
             [slingshot.slingshot :refer [throw+]]
             [puppetlabs.comidi :as comidi]
             [puppetlabs.kitchensink.core :as ks]
-            [puppetlabs.trapperkeeper.services.status.ringutils :as ringutils]
-            [clj-semver.core :as semver]
+            [puppetlabs.ring-middleware.utils :as ringutils]
+            [puppetlabs.ring-middleware.core :as middleware]
             [trptcolin.versioneer.core :as versioneer]
             [clojure.java.jmx :as jmx])
   (:import (java.net URL)
@@ -23,9 +23,14 @@
 (def State
   (schema/enum :running :error :starting :stopping :unknown))
 
+(def Alert
+  {:severity (schema/enum :error :warning :info)
+   :message schema/Str})
+
 (def StatusCallbackResponse
   {:state State
-   :status schema/Any})
+   :status schema/Any
+   (schema/optional-key :alerts) [Alert]})
 
 (def StatusFn (schema/make-fn-schema StatusCallbackResponse ServiceStatusDetailLevel))
 
@@ -41,20 +46,18 @@
 
 ;; this is what gets returned in the HTTP response as json, and thus uses
 ;; underscores rather than hyphens
-;; TODO: merge StatusCallbackResponse with this, rather than duplicating its
-;; two keys, and remove underscores from this schema.
 (def ServiceStatus
   {:service_version schema/Str
    :service_status_version schema/Int
    :state State
    :detail_level ServiceStatusDetailLevel
-   :status schema/Any})
+   :status schema/Any
+   :active_alerts [Alert]})
 
 (def ServicesStatus
   {schema/Str ServiceStatus})
 
-(def SemVerVersion
-  (schema/pred semver/valid-format? "semver"))
+(def Version schema/Str)
 
 (def StatusProxyConfig
   {:proxy-target-url schema/Str
@@ -149,6 +152,8 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Public
 
+(def status-service-name "status-service")
+
 (schema/defn ^:always-validate nominal? :- schema/Bool
   [status :- ServiceStatus]
   (= (:state status) :running))
@@ -158,7 +163,7 @@
   (every? nominal? (vals statuses)))
 
 (schema/defn ^:always-validate
-  get-artifact-version :- SemVerVersion
+  get-artifact-version :- schema/Str
   "Utility function that services can use to get a value to pass in as their
   `service-version` when registering a status callback.  `group-id` and
   `artifact-id` should match the maven/leiningen identifiers for the project
@@ -170,12 +175,6 @@
                (format "Unable to find version number for '%s/%s'"
                  group-id
                  artifact-id))))
-    (when-not (semver/valid-format? version)
-      (throw (IllegalStateException.
-               (format "Service '%s/%s' has version that does not comply with semver: '%s'"
-                 group-id
-                 artifact-id
-                 version))))
     version))
 
 (def status-service-version
@@ -213,6 +212,12 @@
   (let [status-map (service-status-map svc-version status-version status-fn)]
     (swap! status-fns-atom update-in [svc-name] conj status-map)))
 
+(schema/defn ^:always-validate reset-status-context! :- nil
+  "Remove a key from the :status-fns atom in the service context"
+  [status-fns-atom :- clojure.lang.Atom]
+  (reset! status-fns-atom {})
+  nil)
+
 (schema/defn ^:always-validate guarded-status-fn-call :- StatusCallbackResponse
   "Given a status check function, a status detail level, and a timeout in
   seconds, this function calls the status function and handles three types of
@@ -258,11 +263,11 @@
                                      service-status-version)
                                  service)))]
      (if (nil? status)
-       (throw+ {:type    :service-status-version-not-found
-                :message (str "No status function with version "
-                              service-status-version
-                              " found for service "
-                              service-name)})
+       (throw+ {:kind :service-status-version-not-found
+                :msg (str "No status function with version "
+                          service-status-version
+                          " found for service "
+                          service-name)})
        status)))
 
 (schema/defn ^:always-validate get-status-fn :- StatusFn
@@ -273,8 +278,8 @@
    service-status-version :- (schema/maybe schema/Int)]
   (let [service-info (-> services-info-atom deref (get service-name))]
      (if (nil? service-info)
-       (throw+ {:type    :service-info-not-found
-                :message (str "No service info found for service " service-name)})
+       (throw+ {:kind :service-info-not-found
+                :msg (str "No service info found for service " service-name)})
        (:status-fn (matching-service-info service-name service-info service-status-version)))))
 
 (schema/defn ^:always-validate call-status-fn-for-service :- ServiceStatus
@@ -298,12 +303,14 @@
          data (:status callback-resp)
          state (if-not (schema/check State (:state callback-resp))
                  (:state callback-resp)
-                 :unknown)]
+                 :unknown)
+         alerts (get callback-resp :alerts [])]
        {:service_version (:service-version status)
         :service_status_version (:service-status-version status)
         :detail_level level
         :state state
-        :status data})))
+        :status data
+        :active_alerts alerts})))
 
 (schema/defn ^:always-validate call-status-fns :- ServicesStatus
   "Call the latest status function for each service in the service context,
@@ -328,8 +335,7 @@
   (if-let [level (keyword (params :level))]
     (if-not (schema/check ServiceStatusDetailLevel level)
       level
-      (throw+  {:type :request-data-invalid
-                :message (str "Invalid level: " level)}))
+      (ringutils/throw-data-invalid! (str "Invalid level: " level)))
     :info))
 
 (defn get-service-status-version
@@ -339,9 +345,9 @@
   (when-let [level (params :service_status_version)]
     (if-let [parsed-level (ks/parse-int level)]
       parsed-level
-      (throw+ {:type    :request-data-invalid
-               :message (str "Invalid service_status_version. Should be an "
-                          "integer but was " level)}))))
+      (ringutils/throw-data-invalid!
+       (str "Invalid service_status_version. Should be an integer but was "
+            level)))))
 
 (schema/defn ^:always-validate summarize-states :- State
   "Given a map of service statuses, return the 'most severe' state present as
@@ -413,17 +419,17 @@
               (assoc status :service_name service-name)))
           ;; else (no service with that name)
           (ringutils/json-response 404
-             {:type :service-not-found
-              :message (str "No status information found for service "
-                            service-name)})))))
+             {:kind :service-not-found
+              :msg  (str "No status information found for service "
+                         service-name)})))))
 
 (schema/defn ^:always-validate errors-by-type-middleware
   [t :- ringutils/ResponseType]
   (fn [handler]
     (-> handler
-        (ringutils/wrap-request-data-errors t)
-        (ringutils/wrap-schema-errors t)
-        (ringutils/wrap-errors t))))
+        (middleware/wrap-data-errors t)
+        (middleware/wrap-schema-errors t)
+        (middleware/wrap-uncaught-errors t))))
 
 (defn build-handler [path status-fns]
   (comidi/routes->handler
